/* Additional functions used by fvwm and modules to implement the  *
 * pipe transport between them. This code is copyright                    *
 * 1996@Sebastien Marineau <marineau@genie.uottawa.ca>               */

#define INCL_DOSFILEMGR
#define INCL_DOSNMPIPES
#define INCL_DOSERRORS
#include <stdio.h>
#include <os2.h>
#include <sys/fcntl.h>

/* Create a pipe.  dst points to the int 
 which will receive the handles. This is a duplex pipe, which may be used 
  to read/write to the module. Returns 0 on success, -1 on failure */

int fvwm_pipe(int *dst,int temp_index)
{
   HPIPE handle;
   ULONG rc,action;
   char pipename[128];
   int imp_handle;

  sprintf (pipename, "/pipe/fvwm/%.8x.pip", (unsigned)temp_index);

  /* Create the pipe. */

  rc = DosCreateNPipe (pipename, &handle,
                       NP_NOINHERIT | NP_ACCESS_DUPLEX,
                       1 | NP_NOWAIT | NP_TYPE_BYTE | NP_READMODE_BYTE,
                       8192, 8192, 0);
  if (rc != 0)
    return (-1);

 /* Connect to the pipe. */

  rc = DosConnectNPipe (handle);
  if (rc != 0 && rc != ERROR_PIPE_NOT_CONNECTED)
    {
      DosClose (handle);
      return (-1);
    }

  /* Turn on NP_WAIT. */
  /* Leave out for now */

  /* rc = DosSetNPHState (handle, NP_WAIT | NP_READMODE_BYTE);
  if (rc != 0)
    {
      DosClose (handle);
      return (-1);
    } */

/* Now import handle into EMX. */

    imp_handle=_imphandle(handle);
    if (imp_handle < 1){
       dst[0]=dst[1]=-1;
       DosClose(handle);
       return (-1);
       }
    dst[0]=dst[1]=imp_handle;
    fcntl(dst[0],F_SETFD,FD_CLOEXEC);
    return(0);
}

int module_pipe(int *dst,int temp_index)
{
   HPIPE handle;
   ULONG rc,action;
   char pipename[128];
   int imp_handle;

  sprintf (pipename, "/pipe/fvwm/%.8x.pip", (unsigned)temp_index);

  rc = DosOpen (pipename, &handle, &action, 0, FILE_NORMAL,
                OPEN_ACTION_OPEN_IF_EXISTS | OPEN_ACTION_FAIL_IF_NEW,
                OPEN_FLAGS_NOINHERIT | OPEN_ACCESS_READWRITE | OPEN_SHARE_DENYREADWRITE,
                NULL);
  if (rc != 0)
    {
      DosClose (handle);
      return (-1);
    }
  rc = DosSetNPHState(handle,NP_WAIT);
  if (rc != 0)
    {
      DosClose (handle);
      fprintf(stderr,"Bad return code when trying to make pipe blocking! rc = %d\n",rc);
      return (-1);
    }

 

   /* OK, the pipe is open at this end. Let's import the handle */

   imp_handle=_imphandle(handle);
   if(imp_handle<1) {
      dst[0]=dst[1]=-1;
      DosClose(handle);
      return(-1);
      }
    dst[0]=dst[1]=imp_handle;
    fcntl(dst[0],F_SETFD,FD_CLOEXEC);
    return(0);
}

